!  A simple solution to the heat equation using arrays
!  and pointers. Results are plotted with Gnuplot.

program heat

implicit none

integer, parameter             :: nn = 20, &  ! Size of grid
                                  plot_frequency = 10, &
                                  pause_time = 1
real, dimension(nn,nn), target :: plate
real, dimension(nn-2,nn-2)     :: temp
real, pointer, dimension(:,:)  :: n, e, s, w, inside
real, parameter                :: tolerance = 1.0e-3
character(len=20)              :: filename = "heat_data."

real    :: diff
integer :: i, niter, ios

open(unit=11, file="heat.gp", status="replace", &
     action="write", iostat=ios)
if (ios > 0) then
   print *, "Couldn't open heat.gp"
   stop
end if

call system("rm -f heat_data.*")

write(unit=11, fmt="(a)") &
      "set terminal x11", &
      "set pm3d", &
      "set palette", &
      "set ticslevel 0", &
      "set view 0,0"
write(unit=11, fmt="(a, i0)") &
      "pause_seconds=", pause_time

! Set up initial conditions
plate = 0
plate(1:nn,nn) = 1.0  ! boundary values
plate(1,1:nn) = (/ ( 1.0/nn*i, i = 1, nn ) /)

!  Point to parts of the plate
inside => plate(2:nn-1,2:nn-1)
n => plate(1:nn-2,2:nn-1)
s => plate(3:nn,2:nn-1)
e => plate(2:nn-1,3:nn)
w => plate(2:nn-1,1:nn-2)

! Iterate
niter = 0
do
   niter = niter + 1
   temp = (n + e + s + w)/4.0
   diff = maxval(abs(temp-inside))
   inside = temp
   if (modulo(niter, plot_frequency) == 0) then
      call print_data()
   end if
   if (diff < tolerance) exit
end do

if (modulo(niter, plot_frequency) /= 0) then
   call print_data()
end if

close (unit=11, status="keep")

call system("gnuplot -persist ""heat.gp""")

contains

subroutine print_data()
   integer :: i
   write(unit=filename(11:), fmt="(i0)") niter
   open(file=trim(filename), unit=22, status="replace", &
        action="write", iostat=ios)
   if (ios>0) then
      print *, "File open failed:", niter
      stop
   end if
   do i = 1,nn
      write (unit=22, fmt="(999f7.3)") plate(1:nn,i)
   enddo
   close(unit=22, status="keep")
   write (unit=11, fmt="(a)") &
     "splot """ // trim(filename) // """ matrix pt 0", &
     "pause pause_seconds"
end subroutine print_data

end program heat
