module math_module

   integer, parameter, private :: double = &
      selected_real_kind(precision(0.0)+1)

   real, parameter, public :: pi =  &
      3.1415926535897932384626433832795028841972

   real, parameter, public :: e =  &
      2.7182818284590452353602874713526624977572

   real, parameter, public :: gamma =  &
      0.5772156649015328606065120900824024310422

   real, parameter, public :: phi =  &
      1.6180339887498948482045868343656381177203

   real(kind=double), parameter, public :: pi_double =  &
      3.1415926535897932384626433832795028841972_double

   real(kind=double), parameter, public :: e_double =  &
      2.7182818284590452353602874713526624977572_double

   real(kind=double), parameter, public :: gamma_double =  &
      0.5772156649015328606065120900824024310422_double

   real(kind=double), parameter, public :: phi_double =  &
      1.6180339887498948482045868343656381177203_double

   integer, parameter, private :: kind_k = kind(0)

contains

elemental function gcd( a, b) result(gcd_result)

integer( kind= kind_k) :: gcd_result
integer( kind= kind_k), intent( in) :: a, b

!  gcd_result() local

   integer( kind= kind_k) :: a_gcd, b_gcd, rnp1, rn, rnm1

!  gcd_result()

continue                                         ! gcd(), i .gcd. i

! if a or b zero, abs( other) is gcd

   zero_a: if( a == 0_kind_k )then

      gcd_result = abs( b)                         ! note this might return zero

      return                                     ! done

   endif zero_a

   zero_b: if( b == 0_kind_k )then

      gcd_result = abs( a)

      return                                     ! done

   endif zero_b

! set |a| >= |b| ( > 0)
! r1 = a .mod. b
! r0 = b

   a_gcd = max( abs( a), abs( b))                ! a is the greater

   b_gcd = min( abs( a), abs( b))                ! b is the lesser

!  rn = a_gcd .mod. b_gcd                        ! n = 1
   rn = modulo(a_gcd, b_gcd)                       ! n = 1

   rnm1 = b_gcd                                  ! n-1 = 0

! while rn /= 0
!    compute rn+1 = rn .mod. rn-1
! gcd() = rnm1

   zero_rem: do while( rn /= 0_kind_k)           ! til zero remainder

 !     rnp1 = rnm1 .mod. rn                       ! rn+1
      rnp1 = modulo(rnm1, rn)                       ! rn+1

      rnm1 = rn                                  ! n --> n-1

      rn = rnp1                                  ! n+1 --> n

   enddo zero_rem                                ! til zero remainder

   gcd_result = rnm1                               ! done

end function gcd

end module math_module
