module quaternions_module

  intrinsic :: conjg, abs
  public :: quaternion_print, conjg, abs
  public :: operator(+), operator(-), operator(*)
  private :: quat_add, quat_subt, quat_scal_mult, quat_mult, &
       & quat_conjg, quat_abs 

  type, public :: quaternion
     real :: a
     real :: b
     real :: c
     real :: d
  end type quaternion

  interface operator(+)
     module procedure quat_add
  end interface

  interface operator(-)
     module procedure quat_subt
  end interface

  interface operator(*)
     module procedure quat_scal_mult, quat_mult
  end interface

  interface conjg
     module procedure quat_conjg
  end interface

  interface abs
     module procedure quat_abs
  end interface


contains

  subroutine quaternion_print(q)
    type(quaternion), intent(in) :: q
    real :: ap, bp, cp, dp
    ap = q % a
    bp = q % b
    cp = q % c
    dp = q % d
    print "(a1,4f12.6,a1)", "(",ap,bp,cp,dp,")"
  end subroutine quaternion_print

  function quat_add(x,y) result (res)
    type(quaternion), intent(in) :: x, y
    type(quaternion) :: res
    res % a = x % a + y % a
    res % b = x % b + y % b
    res % c = x % c + y % c
    res % d = x % d + y % d
  end function quat_add

  function quat_subt(x,y) result (res)
    type(quaternion), intent(in) :: x, y
    type(quaternion) :: res
    res % a = x % a - y % a
    res % b = x % b - y % b
    res % c = x % c - y % c
    res % d = x % d - y % d
  end function quat_subt

  function quat_scal_mult(c,x) result (res)
    real, intent(in) :: c
    type(quaternion), intent(in) :: x
    type(quaternion) :: res
    res % a = c * (x % a)
    res % b = c * (x % b)
    res % c = c * (x % c)
    res % d = c * (x % d)
  end function quat_scal_mult

  function quat_mult(x,y) result (res)
    type(quaternion), intent(in) :: x, y
    type(quaternion) :: res
    res % a = (x % a) * (y % a) - (x % b) * (y % b) &
         & -(x % c) * (y % c) - (x % d) * (y % d)
    res % b = (x % a) * (y % b) + ( x % b) * ( y % a) &
         & +(x % c) * (y % d) - (x % d) * (y % c)
    res % c = (x % a) * ( y % c) + (x % c) * ( y % a) &
         & + (x % d) * (y % b) - (x % a) * ( y % d)
    res % d = (x % a) * (y % d) + (x % d) * (y % a) &
         & + (x % b) * (y % c) - (x % c) * (y % b)
  end function quat_mult

  function quat_conjg(x) result (res)
    type(quaternion), intent(in) :: x
    type(quaternion) :: res
    res % a = x % a
    res % b = -(x % b)
    res % c = -(x % c)
    res % d = -(x % d)
  end function quat_conjg

  function quat_abs(x) result (res)
    type(quaternion), intent(in) :: x
    real :: res
    res=sqrt((x % a)**2+(x % b)**2 + (x % c)**2 + (x % d)**2)
  end function quat_abs

end module quaternions_module

!program Quaternions
!  use Quaternions_needs
!  type(quaternion) :: u, v
!  u=quaternion(1,2,3,4)
!  v=quaternion(5,6,7,8)
!  call quaternion_print(u+v)
!  print *, 3+4
!  print *
!  call quaternion_print(u-v)
!  print *, 3-4
!  print *
!  call quaternion_print(3.0*u)
!  call quaternion_print(u*v)
!  print *, 3*4
!  print *
!  call quaternion_print(conjg(u))
!  print *, conjg((3,4))
!  print *
!  print *, (abs(u))
!  print *, abs((3,4))
!end program quaternions_module
