! bof
! **********************************************************************
! Source Control:

! $Id: typerat.f90 1.6 2000/05/24 15:30:55Z Dan Release $

! **********************************************************************
!  copyright 2000 Purple Sage Computing Solutions, Inc.

!   This library is free software; you can redistribute it and/or
!   modify it under the terms of the GNU Library General Public
!   License as published by the Free Software Foundation; either
!   version 2 of the License, or (at your option) any later version.
 
!   This library is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
!   Library General Public License for more details.
 
!   You should have received a copy of the GNU Library General Public
!   License along with this library; if not, write to the Free
!   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
! To report bugs, suggest enhancements, etc. to the Authors,
! Contact:
!    Purple Sage Computing Solutions, Inc.
!                               send email to dnagle@erols.com
!                                   or fax to 703 471 0684 (USA)
!                                  or mail to 12142 Purple Sage Ct.
!                                             Reston, VA 20194-5621 USA

! **********************************************************************
! Fortran 90 module type_rational

! **********************************************************************
! a rational number is the ratio of two integers

! **********************************************************************
! use standard_types

! **********************************************************************

!  type_rational types

!     rational two int_k integers, numerator & denominator

!  type_rational operators

!     =

!     + unary rational
!     - negation

!     + binary compute rational
!     -
!     *
!     /

!     .eq. binary compare rational
!     .ne.
!     .lt.
!     .le.
!     .ge.
!     .gt.

!     .inverse. exchange numerator & denominator
!     .reduce. reduce to lowest terms

!  type_rational library

!     rat() from int

!     int() from rational
!     nint() from rational
!     real() from rational

!     ceiling()
!     floor()

!     sign() of a rational
!     abs()

!     epsilon() for type rational
!     huge()
!     tiny()

!     numerator() numerator
!     denominator() denominator

!     is_finite() true if ( x / y), y /= 0
!     is_infinity() true if ( x / 0), x /= 0
!     is_nan() true if ( 0 / 0)

!     inverse() exchange numerator & denominator
!     reduce() reduce to lowest terms

!     bit_size() extend bit_size() to type rational
!     swap() extend swap() to type rational

! **********************************************************************

!  type_rational

! **********************************************************************

module rationals_module

! **********************************************************************

!  use usual description of processor

!use standard_types

! **********************************************************************

!  use usual functions

!use standard_functions

! **********************************************************************

!  declare all variables

implicit none

! **********************************************************************

!F

integer, parameter :: int_k = kind(0), &
                      byte_k = selected_int_kind(1), &
                      short_k = selected_int_kind(4), &
!!!!!!!!!!!!!                      long_k = selected_int_kind(11), &
                      double_k = selected_real_kind(precision(0.0)+1)

!interface operator(.gcd.)
!elemental function gcd(a, b) result(gcd_result)
!integer, parameter :: int_k = kind(0)
!integer( kind= int_k) :: gcd_result
!integer( kind= int_k), intent( in) :: a, b
!end function gcd
!end interface

!  type_rational rcs strings

character( len= *), parameter :: type_rational_rcs_id = &
   '$Id: typerat.f90 1.6 2000/05/24 15:30:55Z Dan Release $'

! **********************************************************************

!  type_rational

type :: rational

   private                                                 ! access via library

   integer( kind = int_k) :: n                             ! numerator
   integer( kind = int_k) :: d                             ! denominator

end type rational

! **********************************************************************

!  library

! **********************************************************************

!  assignment for type rational

private :: rationalo_int                                 ! specific

interface assignment( =)
   module procedure rationalo_int
end interface

private :: int_to_rational                                 ! specific

interface assignment( =)
   module procedure int_to_rational
end interface

! **********************************************************************

!  +, -, unary operators for rational

private :: rational_plus                                   ! specific

interface operator( +)
   module procedure rational_plus
end interface

private :: rational_minus                                  ! specific

interface operator( -)
   module procedure rational_minus
end interface

! **********************************************************************

!  +, -, *, / binary operators for rational

private :: rational_add                                    ! specific

interface operator( +)
   module procedure rational_add
end interface

private :: rational_sub                                    ! specific

interface operator( -)
   module procedure rational_sub
end interface

private :: rational_mul                                    ! specific

interface operator( *)
   module procedure rational_mul
end interface

private :: rational_div                                    ! specific

interface operator( /)
   module procedure rational_div
end interface

! **********************************************************************

!  ==, /=, <=, <, >, >= operators for rational

private :: rational_eq                                     ! specific

interface operator( ==)
   module procedure rational_eq
end interface

private :: rational_ne                                     ! specific

interface operator( /=)
   module procedure rational_ne
end interface

private :: rational_lt                                     ! specific

interface operator( <)
   module procedure rational_lt
end interface

private :: rational_le                                     ! specific

interface operator( <=)
   module procedure rational_le
end interface

private :: rational_ge                                     ! specific

interface operator( >=)
   module procedure rational_ge
end interface

private :: rational_gt                                     ! specific

interface operator( >)
   module procedure rational_gt
end interface

! **********************************************************************

!  rat() for type rational

public :: rat                                              ! use generic

private :: byte_rat                                        ! specific
private :: short_rat                                       ! specific
private :: int_rat                                         ! specific
!F private :: long_rat                                        ! specific

interface rat
   module procedure byte_rat
   module procedure short_rat
   module procedure int_rat
!F    module procedure long_rat
end interface

! **********************************************************************

!  int() for type rational

intrinsic :: int                                           ! extend intrinsic

public :: int                                              ! use generic

private :: rational_int                                    ! specific

interface int
   module procedure rational_int
end interface

! **********************************************************************

!  nint() for type rational

intrinsic :: nint                                          ! extend intrinsic

public :: nint                                             ! use generic

private :: rational_nint                                   ! specific

interface nint
   module procedure rational_nint
end interface

! **********************************************************************

!  real() for type rational

intrinsic :: real                                          ! extend intrinsic

public :: real                                             ! use generic

private :: rational_real                                   ! specific

interface real
   module procedure rational_real
end interface

! **********************************************************************

!  sign() for type rational

intrinsic :: sign                                          ! extend intrinsic

public :: sign                                             ! use generic

private :: rational_sign                                   ! specific

interface sign
   module procedure rational_sign
end interface

! **********************************************************************

!  abs() for type rational

intrinsic :: abs                                           ! extend intrinsic

public :: abs                                              ! use generic

private :: rational_abs                                    ! specific

interface abs
   module procedure rational_abs
end interface

! **********************************************************************

!  huge() for type rational

intrinsic :: huge                                          ! extend intrinsic

public :: huge                                             ! use generic

private :: rational_huge                                   ! specific

interface huge
   module procedure rational_huge
end interface

! **********************************************************************

!  tiny() for type rational

intrinsic :: tiny                                          ! extend intrinsic

public :: tiny                                             ! use generic

private :: rationaliny                                   ! specific

interface tiny
   module procedure rationaliny
end interface

! **********************************************************************

!  epsilon() for type rational

intrinsic :: epsilon                                       ! extend intrinsic

public :: epsilon                                          ! use generic

private :: rational_epsilon                                ! specific

interface epsilon
   module procedure rational_epsilon
end interface

! **********************************************************************

!  numerator() for type rational

public :: numerator                                        ! use generic

private :: rational_numerator                              ! specific

interface numerator
   module procedure rational_numerator
end interface

!  denominator() for type rational

public :: denominator                                      ! use generic

private :: rational_denominator                            ! specific

interface denominator
   module procedure rational_denominator
end interface

! **********************************************************************

!  is_finite() for type rational

public :: is_finite                                        ! use generic

private :: rational_is_finite                              ! specific

interface is_finite
   module procedure rational_is_finite
end interface

! **********************************************************************

!  is_infinity() for type rational

public :: is_infinity                                      ! use generic

private :: rational_is_infinity                            ! specific

interface is_infinity
   module procedure rational_is_infinity
end interface

! **********************************************************************

!  is_nan() for type rational

public :: is_nan                                           ! use generic

private :: rational_is_nan                                 ! specific

interface is_nan
   module procedure rational_is_nan
end interface

! **********************************************************************

!  inverse() for type rational

public :: inverse                                          ! use generic

private :: rational_inverse                                ! specific

interface operator( .inverse.)
   module procedure rational_inverse
end interface

interface inverse
   module procedure rational_inverse
end interface

! **********************************************************************

!  reduce() for type rational

public :: reduce                                           ! use generic

private :: rational_reduce                                 ! specific

interface operator( .reduce.)
   module procedure rational_reduce
end interface

interface reduce
   module procedure rational_reduce
end interface

! **********************************************************************

!  bit_size() for type rational

intrinsic :: bit_size                                      ! extend intrinsic

public :: bit_size                                         ! use generic

private :: rational_bit_size                               ! specific

interface bit_size
   module procedure rational_bit_size
end interface

! **********************************************************************

!  swap() for type rational

public :: swap                                             ! use generic

private :: rational_swap                                   ! specific

interface swap
   module procedure rational_swap
end interface

! **********************************************************************

!  module procedures

! **********************************************************************

contains

! **********************************************************************

!  rationalo_int(): assign i( 1: 2) = r

subroutine rationalo_int( b, a)

type( rational), intent( in) :: a

integer( kind= int_k), dimension( 2), intent( out) :: b

!  rationalo_int()

continue                                         ! =

   b = (/ a% n, a% d /)

return                                           ! =

!  rationalo_int()

end subroutine rationalo_int

! ----------------------------------------------------------------------

!  int_to_rational(): assign r = i( 1: 2)

subroutine int_to_rational( b, a)

integer( kind= int_k), dimension( 2), intent( in) :: a

type( rational), intent( out) :: b

!  int_to_rational()

continue                                         ! =

   b = rational( a( 1), a( 2) )

return                                           ! =

!  int_to_rational()

end subroutine int_to_rational

! **********************************************************************

!  rational_plus(): unary + rational

elemental type( rational) function rational_plus( a)

type( rational), intent( in) :: a

!  rational_plus()

continue                                         ! +

   rational_plus = rational( a% n, a% d)

return                                           ! +

!  rational_plus()

end function rational_plus

! ----------------------------------------------------------------------

!  rational_minus(): unary - rational

elemental type( rational) function rational_minus( a)

type( rational), intent( in) :: a

!  rational_minus()

continue                                         ! -

   rational_minus = rational( -a% n , a% d)

return                                           ! -

!  rational_minus()

end function rational_minus

! **********************************************************************

!  rational_add(): add two rationals

elemental type( rational) function rational_add( a, b)

type( rational), intent( in) :: a, b

!  rational_add()

continue                                         ! +

   rational_add = rational( ( a% n * b% d) + ( b% n * a% d), a% d * b% d)

return                                           ! +

!  rational_add()

end function rational_add

! ----------------------------------------------------------------------

!  rational_sub(): subtract two rationals

elemental type( rational) function rational_sub( a, b)

type( rational), intent( in) :: a, b

!  rational_sub()

continue                                         ! -

   rational_sub = rational( ( a% n * b% d) - ( b% n * a% d), a% d * b% d)

return                                           ! -

!  rational_sub()

end function rational_sub

! ----------------------------------------------------------------------

!  rational_mul(): multiply two rationals

elemental type( rational) function rational_mul( a, b)

type( rational), intent( in) :: a, b

!  rational_mul()

continue                                         ! *

   rational_mul = rational( a% n * b% n, a% d * b% d)

return                                           ! *

!  rational_mul()

end function rational_mul

! ----------------------------------------------------------------------

!  rational_div(): divide two rationals

elemental type( rational) function rational_div( a, b)

type( rational), intent( in) :: a, b

!  rational_div()

continue                                         ! /

   rational_div = rational( a% n * b% d, a% d * b% n)

return                                           ! /

!  rational_div()

end function rational_div

! **********************************************************************

!  rational_eq(): eq two rationals

elemental logical function rational_eq( a, b)

type( rational), intent( in) :: a, b

!  rational_eq() local

   type( rational) :: ra, rb

!  rational_eq()

continue                                         ! ==

   ra = reduce( a)

   rb = reduce( b)

   rational_eq = ( ra% n == rb% n) .and. ( ra% d == rb% d)

return                                           ! ==

!  rational_eq()

end function rational_eq

! ----------------------------------------------------------------------

!  rational_ne(): ne two rationals

elemental logical function rational_ne( a, b)

type( rational), intent( in) :: a, b

!  rational_ne() local

   type( rational) :: ra, rb

!  rational_ne()

continue                                         ! /=

   ra = reduce( a)

   rb = reduce( b)

   rational_ne = ( ra% n /= rb% n) .or. ( ra% d /= rb% d)

return                                           ! /=

!  rational_ne()

end function rational_ne

! ----------------------------------------------------------------------

!  rational_lt(): lt two rationals

elemental logical function rational_lt( a, b)

type( rational), intent( in) :: a, b

!  rational_lt()

continue                                         ! <

   rational_lt = sign( 1_int_k, a - b) < 0

return                                           ! <

!  rational_lt()

end function rational_lt

! ----------------------------------------------------------------------

!  rational_le(): le two rationals

elemental logical function rational_le( a, b)

type( rational), intent( in) :: a, b

!  rational_le()

continue                                         ! <=

   rational_le = sign( 1_int_k, a - b) < 0 &
                 .or. a .eq. b

return                                           ! <=

!  rational_le()

end function rational_le

! ----------------------------------------------------------------------

!  rational_ge(): ge two rationals

elemental logical function rational_ge( a, b)

type( rational), intent( in) :: a, b

!  rational_ge()

continue                                         ! >=

   rational_ge = sign( 1_int_k, a - b) > 0 &
                 .or. a .eq. b

return                                           ! >=

!  rational_ge()

end function rational_ge

! ----------------------------------------------------------------------

!  rational_gt(): gt two rationals

elemental logical function rational_gt( a, b)

type( rational), intent( in) :: a, b

!  rational_gt()

continue                                         ! >

   rational_gt = sign( 1_int_k, a - b) > 0

return                                           ! >

!  rational_gt()

end function rational_gt

! **********************************************************************

!  byte_rat(): rational from integer

elemental type( rational) function byte_rat( i, j)

integer( kind= byte_k), intent( in) :: i

integer( kind= byte_k), optional, intent( in) :: j

!  byte_rat()

continue                                                   ! rat()

   one_two: if( present( j) )then

      byte_rat = rational( int( i, kind= int_k), int( j, kind= int_k))

   else one_two

      byte_rat = rational( int( i, kind= int_k), 1_int_k)

   endif one_two

return                                                     ! rat()

!  byte_rat()

end function byte_rat

! ----------------------------------------------------------------------

!  short_rat(): rational from integer

elemental type( rational) function short_rat( i, j)

integer( kind= short_k), intent( in) :: i

integer( kind= short_k), optional, intent( in) :: j

!  short_rat()

continue                                                   ! rat()

   one_two: if( present( j) )then

      short_rat = rational( int( i, kind= int_k), int( j, kind= int_k))

   else one_two

      short_rat = rational( int( i, kind= int_k), 1_int_k)

   endif one_two

return                                                     ! rat()

!  short_rat()

end function short_rat

! ----------------------------------------------------------------------

!  int_rat(): rational from integer

elemental type( rational) function int_rat( i, j)

integer( kind= int_k), intent( in) :: i

integer( kind= int_k), optional, intent( in) :: j

!  int_rat()

continue                                                   ! rat()

   one_two: if( present( j) )then

      int_rat = rational( i, j)

   else one_two

      int_rat = rational( i, 1_int_k)

   endif one_two

return                                                     ! rat()

!  int_rat()

end function int_rat

!F ! ----------------------------------------------------------------------
!F 
!F !  long_rat(): rational from integer
!F 
!F elemental type( rational) function long_rat( i, j)
!F 
!F integer( kind= long_k), intent( in) :: i
!F 
!F integer( kind= long_k), optional, intent( in) :: j
!F 
!F !  long_rat()
!F 
!F continue                                                   ! rat()
!F 
!F    one_two: if( present( j) )then
!F 
!F       long_rat = rational( int( i, kind= int_k), int( j, kind= int_k))
!F 
!F    else one_two
!F 
!F       long_rat = rational( int( i, kind= int_k), 1_int_k)
!F 
!F    endif one_two
!F 
!F return                                                     ! rat()
!F 
!F !  long_rat()
!F 
!F end function long_rat

! **********************************************************************

!  rational_int(): integer from rational

elemental integer( kind= int_k) function rational_int( r)

type( rational), intent( in) :: r

!  rational_int()

continue                                                   ! int()

   rational_int = int( real( r% n, kind= double_k) / real( r% d, kind= double_k) )

return                                                     ! int()

!  rational_int()

end function rational_int

! ----------------------------------------------------------------------

!  rational_nint(): integer from rational

elemental integer( kind= int_k) function rational_nint( r)

type( rational), intent( in) :: r

!  rational_nint()

continue                                                   ! nint()

   rational_nint = nint( real( r% n, kind= double_k) / real( r% d, kind= double_k) )

return                                                     ! nint()

!  rational_nint()

end function rational_nint

! ----------------------------------------------------------------------

!  rational_real(): integer from rational

elemental real( kind= double_k) function rational_real( r)

type( rational), intent( in) :: r

!  rational_real()

continue                                                   ! real()

   rational_real = real( r% n, kind= double_k) / real( r% d, kind= double_k)

return                                                     ! real()

!  rational_real()

end function rational_real

! **********************************************************************

!  rational_sign(): integer from rational

elemental integer( kind= int_k) function rational_sign( i, r)

integer( kind= int_k), intent( in) :: i

type( rational), intent( in) :: r

!  rational_sign()

continue                                                   ! sign()

   plus_minus: if( sign( 1_int_k, r% n) * sign( 1_int_k, r% d) > 0 )then

      rational_sign = abs( i)                              ! positive integer

   else plus_minus

      rational_sign = -abs( i)                             ! negative integer

   endif plus_minus

return                                                     ! sign()

!  rational_sign()

end function rational_sign

! **********************************************************************

!  rational_abs(): extend abs() to type rational

elemental type( rational) function rational_abs( r)

type( rational), intent( in) :: r

!  rational_abs()

continue                                                   ! abs()

   rational_abs = rational( abs( r% n), abs( r% d))

return                                                     ! abs()

!  rational_abs()

end function rational_abs

! **********************************************************************

!  rational_huge(): integer from rational

elemental type( rational) function rational_huge( r)

type( rational), intent( in) :: r

!  rational_huge()

continue                                                   ! huge()

   rational_huge = rational( huge( 0_int_k), 1_int_k)

return                                                     ! huge()

!  rational_huge()

end function rational_huge

! **********************************************************************

!  rationaliny(): extend tiny() to type rational

elemental type( rational) function rationaliny( r)

type( rational), intent( in) :: r

!  rationaliny()

continue                                                   ! tiny()

   rationaliny = rational( 1_int_k, huge( 0_int_k))

return                                                     ! tiny()

!  rationaliny()

end function rationaliny

! **********************************************************************

!  rational_epsilon(): extend epsilon() to type rational

elemental type( rational) function rational_epsilon( r)

type( rational), intent( in) :: r

!  rational_epsilon()

continue                                                   ! epsilon()

   rational_epsilon = rational( 1_int_k, ( huge( 0_int_k) - 1_int_k))

return                                                     ! epsilon()

!  rational_epsilon()

end function rational_epsilon

! **********************************************************************

!  rational_numerator(): return numerator

elemental integer( kind= int_k) function rational_numerator( r)

type( rational), intent( in) :: r

!  rational_numerator()

continue                                                   ! numerator()

   rational_numerator = r% n

return                                                     ! numerator()

!  rational_numerator()

end function rational_numerator

! **********************************************************************

!  rational_denominator(): return denominator

elemental integer( kind= int_k) function rational_denominator( r)

type( rational), intent( in) :: r

!  rational_denominator()

continue                                                   ! denominator()

   rational_denominator = r% d

return                                                     ! denominator()

!  rational_denominator()

end function rational_denominator

! **********************************************************************

!  rational_is_finite(): true if its argument is x/0, x/=0

elemental logical function rational_is_finite( r)

type( rational), intent( in) :: r

!  rational_is_finite()

continue                                                   ! is_finite()

   rational_is_finite = r% d /= 0_int_k

return                                                     ! is_finite()

!  rational_is_finite()

end function rational_is_finite

! **********************************************************************

!  rational_is_infinity(): true if its argument is x/0, x/=0

elemental logical function rational_is_infinity( r)

type( rational), intent( in) :: r

!  rational_is_infinity()

continue                                                   ! is_infinity()

   rational_is_infinity = ( r% n /= 0_int_k) .and. ( r% d == 0_int_k)

return                                                     ! is_infinity()

!  rational_is_infinity()

end function rational_is_infinity

! **********************************************************************

!  rational_is_nan(): true if its argument is 0/0

elemental logical function rational_is_nan( r)

type( rational), intent( in) :: r

!  rational_is_nan()

continue                                                   ! is_nan()

   rational_is_nan = ( r% n == 0_int_k) .and. ( r% d == 0_int_k)

return                                                     ! is_nan()

!  rational_is_nan()

end function rational_is_nan

! **********************************************************************

!  rational_inverse(): extend inverse() to type rational

elemental type( rational) function rational_inverse( r)

type( rational), intent( in) :: r

!  rational_inverse()

continue                                                   ! .inverse., inverse()

   rational_inverse = rational( r% d, r% n)

return                                                     ! .inverse., inverse()

!  rational_inverse()

end function rational_inverse

! **********************************************************************

!  rational_reduce(): extend reduce() to type rational

elemental type( rational) function rational_reduce( r)

type( rational), intent( in) :: r

!  rational_reduce() local

   integer( kind= int_k) :: div

!  rational_reduce()

continue                                                   ! .reduce., reduce()

!   div = r% n .gcd. r% d                                   ! greatest common divisor
   div = gcd(r% n, r% d)                                   ! greatest common divisor

   rational_reduce = rational( r% n / div, r% d / div)

return                                                     ! .reduce., reduce()

!  rational_reduce()

end function rational_reduce

! **********************************************************************

!  rational_bit_size(): extend bit_size() to type rational

integer function rational_bit_size( r)

type( rational), intent( in) :: r

!  rational_bit_size()

continue                                                   ! bit_size()

   rational_bit_size = 2 * bit_size( r% n)                 ! 2 int integers

return                                                     ! bit_size()

!  rational_bit_size()

end function rational_bit_size

! **********************************************************************

!  rational_swap(): extend swap() to type rational

subroutine rational_swap( a, b)

type( rational), intent( inout) :: a, b

!  rational_swap() local

   type( rational) :: t1, t2

!  rational_swap()

continue                                                   ! swap()

   t1 = a

   t2 = b

   b = t1

   a = t2

return                                                     ! swap()

!  rational_swap()

end subroutine rational_swap

! **********************************************************************

!  type_rational

! $Id: typerat.f90 1.6 2000/05/24 15:30:55Z Dan Release $
! **********************************************************************

elemental function gcd( a, b) result(gcd_result)

integer, parameter :: kind_k = kind(0)
integer( kind= kind_k) :: gcd_result
integer( kind= kind_k), intent( in) :: a, b

!  gcd_result() local

   integer( kind= kind_k) :: a_gcd, b_gcd, rnp1, rn, rnm1

!  gcd_result()

continue                                         ! gcd(), i .gcd. i

! if a or b zero, abs( other) is gcd

   zero_a: if( a == 0_kind_k )then

      gcd_result = abs( b)                         ! note this might return zero

      return                                     ! done

   endif zero_a

   zero_b: if( b == 0_kind_k )then

      gcd_result = abs( a)

      return                                     ! done

   endif zero_b

! set |a| >= |b| ( > 0)
! r1 = a .mod. b
! r0 = b

   a_gcd = max( abs( a), abs( b))                ! a is the greater

   b_gcd = min( abs( a), abs( b))                ! b is the lesser

!  rn = a_gcd .mod. b_gcd                        ! n = 1
   rn = modulo(a_gcd, b_gcd)                       ! n = 1

   rnm1 = b_gcd                                  ! n-1 = 0

! while rn /= 0
!    compute rn+1 = rn .mod. rn-1
! gcd() = rnm1

   zero_rem: do while( rn /= 0_kind_k)           ! til zero remainder

 !     rnp1 = rnm1 .mod. rn                       ! rn+1
      rnp1 = modulo(rnm1, rn)                       ! rn+1

      rnm1 = rn                                  ! n --> n-1

      rn = rnp1                                  ! n+1 --> n

   enddo zero_rem                                ! til zero remainder

   gcd_result = rnm1                               ! done

end function gcd

end module rationals_module


!program test_rat
!use rationals_module
!type(rational) :: r1, r2
!
!r1=(/3,4/)
!r2=(/5,6/)
!r1=r1+r2
!print *, real(r1)
!end program test_rat
